// Copyright (c) 2022 Emma Tebibyte
// SPDX-License-Identifier: AGPL-3.0-or-later

/* Tomcat is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * Tomcat is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::env;
use std::fs::File;
use std::io::Read;
use std::path::Path;
use std::process;

use toml::Value;

fn main() {
  let mut arguments: Vec<String> = env::args().collect();
  let argv0 = arguments.remove(0);
  if arguments[1].is_empty() {
    println!("Usage: {} [table...].[value] [file...]", argv0);
    process::exit(64); // sysexits(3) EX_USAGE
  }
  let input = &arguments[1];
  let mut content = String::new();
  
  let file = Path::new(&input);
  if file.is_file() {
    File::open(file).unwrap().read_to_string(&mut content).unwrap();
  } else { content = input.to_string(); }

  let tabkey: Vec<&str> = arguments[0].split(".").collect();
  let mut root = content.parse::<Value>().unwrap();
  
  let mut out = String::new();
  let mut valiter = tabkey.iter().peekable();
  
  while let Some(item) = valiter.next() {
    match root.get(item) {
      Some(value) => {
        match value {
          // TODO: Implement other type parsing
          Value::Array(_array) => {}, 
          Value::Boolean(_boolean) => {}, 
          Value::Datetime(_datetime) => {}, 
          Value::Float(_float) => {},
          Value::Integer(_int) => {},
          Value::String(string) => {
            match valiter.peek() {
              Some(next) => {
                println!("{}: {}: Not a table.", argv0, item);
                process::exit(65); // sysexits(3) EX_DATAERR
              },
              None => out.push_str(string.as_str()),
            };
            break
          }
          Value::Table(table) => root = toml::Value::Table(table.to_owned()),
        };
      },
      None => {
        println!("{}: {}: No such table or key.", argv0, item);
        process::exit(65); // sysexits(3) EX_DATAERR
      },
    };
  }
  println!("{}", out);
}
